@@ -45,7 +45,7 @@ class KWArgDict(dict):
 		else:
 			return dict.__getitem__(self, key)
 
-class API:
+class API(object):
 	def __init__(self, consumer_key, consumer_secret, access_token = None, access_token_secret = None, host = "api.twitter.com", root = "/1.1/", oauth_host = "api.twitter.com", oauth_root = "/oauth/", secure = True, timeout = None):
 		self.consumer_key = consumer_key
 		self.consumer_secret = consumer_secret
@@ -59,9 +59,13 @@ class API:
 		self.timeout = timeout
 		self._endpoint = None
 		self._multipart = False
+		self.request_token = None
+		self.request_token_secret = None
 		self.user = self.verify_credentials() if self.access_token and self.access_token_secret else None
 	
 	def __getattr__(self, attr):
+		if attr.startswith("__"):
+			return object.__getattr__(self, attr)
 		self._endpoint = attr
 		return self.api_call
 	
@@ -70,6 +74,10 @@ class API:
 		self.access_token_secret = access_token_secret
 		self.user = self.verify_credentials()
 	
+	def set_request_token(self, request_token, request_token_secret):
+		self.request_token = request_token
+		self.request_token_secret = request_token_secret
+	
 	def parse_qs(self, qs):
 		return dict([(key, values[0]) for key, values in urlparse.parse_qs(qs).iteritems()])
 	
@@ -84,7 +92,9 @@ class API:
 			'oauth_timestamp': str(int(time.time())),
 			'oauth_version': "1.0",
 		}
-		if self.access_token:
+		if self.request_token:
+			auth_data['oauth_token'] = self.request_token
+		elif self.access_token:
 			auth_data['oauth_token'] = self.access_token
 		return auth_data
 	
@@ -110,7 +120,12 @@ class API:
 		signature_base.append(urllib.quote(url, safe = "~"))
 		signature_base.append(urllib.quote(param_string, safe = "~"))
 		signature_base = "&".join(signature_base)
-		token_secret = urllib.quote(self.access_token_secret, safe = "~") if self.access_token_secret else ""
+		if self.request_token:
+			token_secret = urllib.quote(self.request_token_secret, safe = "~")
+		elif self.access_token:
+			token_secret = urllib.quote(self.access_token_secret, safe = "~")
+		else:
+			token_secret = ""
 		signing_key = "&".join([urllib.quote(self.consumer_secret, safe = "~"), token_secret])
 		signature = hmac.new(signing_key, signature_base, hashlib.sha1)
 		signature = urllib.quote(binascii.b2a_base64(signature.digest())[:-1], safe = "~")
@@ -161,9 +176,8 @@ class API:
 		url = self.build_request_url(self.oauth_root, 'request_token')
 		resp = self.do_request("GET", url, is_json = False)
 		token_data = self.parse_qs(resp)
-		self.access_token = token_data['oauth_token']
-		self.access_token_secret = token_data['oauth_token_secret']
-		return (self.access_token, self.access_token_secret)
+		self.set_request_token(token_data['oauth_token'], token_data['oauth_token_secret'])
+		return (self.request_token, self.request_token_secret)
 	
 	def get_auth_url(self, token = None):
 		if token is None:
@@ -172,23 +186,27 @@ class API:
 	
 	def authenticate(self, verifier):
 		url = self.build_request_url(self.oauth_root, 'access_token')
-		resp = self.do_request("GET", url, is_json = False)
+		resp = self.do_request("POST", url, post = {'oauth_verifier': verifier}, is_json = False)
 		token_data = self.parse_qs(resp)
-		self.access_token = token_data['oauth_token']
-		self.access_token_secret = token_data['oauth_token_secret']
-		self.user = self.verify_credentials()
+		self.set_access_token(token_data['oauth_token'], token_data['oauth_token_secret'])
 		return ((self.access_token, self.access_token_secret), token_data['user_id'], token_data['screen_name'])
 	
 	def parse_param(self, key, value):
 		if type(value) == bool:
 			value = "true" if value else "false"
+		elif type(value) == list:
+			value = ",".join([str(val) for val in value])
+		elif type(value) != str and value is not None:
+			value = str(value)
+		if type(key) != str:
+			key = str(key)
 		return (key, value)
 	
 	def parse_params(self, params):
 		files = {}
 		_params = dict(params.items())
 		for key, value in params.iteritems():
-			if value is None:
+			if value in [None, []]:
 				del _params[key]
 			if key == 'image' or key == 'media' or key == 'banner':
 				del _params[key]
@@ -294,7 +312,7 @@ class API:
 		
 		resp = self.do_request("POST" if data['post'] else "GET", url, get_data, post_data, files, stream = stream)
 		if stream:
-			for line in resp.iter_lines(chunk_size = 64):
+			for line in resp.iter_lines(chunk_size = 1):
 				if not line:
 					continue
 				entity = self.parse_stream_entity(line)
