@@ -1,9 +1,16 @@
 # Copyright (C) 2013 Julian Metzler
 # See the LICENSE file for the full license.
 
+import locale
 import utils
 from datetime import datetime
 
+def strptime(string, fmt = '%a %b %d %H:%M:%S +0000 %Y'):
+	locale.setlocale(locale.LC_TIME, 'C')
+	value = datetime.strptime(string, fmt)
+	locale.setlocale(locale.LC_TIME, '')
+	return value
+
 class PseudoAPI:
 	def __getattr__(self, name):
 		raise NotImplementedError("This model does not have an API instance associated with it.")
@@ -25,13 +32,19 @@ class AttrDict(dict):
 				self[key] = value
 	
 	def __getattr__(self, name):
-		return self.__getitem__(name)
+		try:
+			return self.__getitem__(name)
+		except:
+			return dict.__getattr__(self, name)
 
 class Model(AttrDict):
 	api = PseudoAPI()
 	
 	def __getattr__(self, name):
-		return self.__getitem__(name)
+		try:
+			return self.__getitem__(name)
+		except:
+			return AttrDict.__getattr__(self, name)
 	
 	@classmethod
 	def from_json(cls, data):
@@ -87,7 +100,7 @@ class Status(Model):
 		tmp = cls()
 		for key, value in self.iteritems():
 			if key == 'created_at':
-				value = datetime.strptime(value, '%a %b %d %H:%M:%S +0000 %Y')
+				value = strptime(value)
 			elif key == 'user':
 				value = User.from_json(value)
 			elif key == 'source':
@@ -128,7 +141,7 @@ class User(Model):
 		self = cls(Model.from_json(data))
 		for key, value in self.iteritems():
 			if key == 'created_at':
-				value = datetime.strptime(value, '%a %b %d %H:%M:%S +0000 %Y')
+				value = strptime(value)
 			elif key == 'status':
 				value = Status.from_json(value)
 			self[key] = value
@@ -198,7 +211,7 @@ class Message(Model):
 		self = cls(Model.from_json(data))
 		for key, value in self.iteritems():
 			if key == 'created_at':
-				value = datetime.strptime(value, '%a %b %d %H:%M:%S +0000 %Y')
+				value = strptime(value)
 			elif key == 'sender' or key == 'receiver':
 				value = User.from_json(value)
 			self[key] = value
@@ -214,6 +227,12 @@ class OEmbed(Model):
 	pass
 
 class Relationship(Model):
+	@classmethod
+	def from_json(cls, data):
+		self = cls(Model.from_json(data['relationship']))
+		return self
+
+class SimpleRelationship(Model):
 	@classmethod
 	def from_json(cls, data):
 		self = cls(Model.from_json(data))
@@ -239,7 +258,7 @@ class List(Model):
 		self = cls(Model.from_json(data))
 		for key, value in self.iteritems():
 			if key == 'created_at':
-				value = datetime.strptime(value, '%a %b %d %H:%M:%S +0000 %Y')
+				value = strptime(value)
 			elif key == 'user':
 				value = User.from_json(value)
 			self[key] = value
@@ -269,7 +288,7 @@ class SavedSearch(Model):
 		self = cls(Model.from_json(data))
 		for key, value in self.iteritems():
 			if key == 'created_at':
-				value = datetime.strptime(value, '%a %b %d %H:%M:%S +0000 %Y')
+				value = strptime(value)
 			self[key] = value
 		return self
 	
@@ -398,6 +417,9 @@ class IDCollection(list):
 class RelationshipCollection(ModelCollection):
 	model = Relationship
 
+class SimpleRelationshipCollection(ModelCollection):
+	model = SimpleRelationship
+
 class ListCollection(ModelCollection):
 	model = List
 
